@@ -164,8 +164,8 @@ class MainWidget(qt.QMainWindow):
         
         self.patchColors = {'std': 'black', 'new': '#009600', 'remove': '#C80000', 'head': '#C800C8'}
 
-        self.files = []
-
+        self.files = scm.fileSetFactory(lambda f: self.addFile(f),
+                                        lambda f: self.removeFile(f))
         f = File()
         f.text = "Commit message"
         f.textW = self.newTextEdit()
@@ -183,11 +183,19 @@ class MainWidget(qt.QMainWindow):
 
     def updateStatusBar(self):
         if not self.cmitFile.textW.isVisible():
-            self.statusBar().clear()
+            self.setStatusBar('')
+
+    def setStatusBar(self, string):
+        branch = scm.getCurrentBranch()
+        if branch:
+            prefix = '[' + branch + '] '
+        else:
+            prefix = ''
+        self.statusBar().message(prefix + string)
 
     def updateCommitCursor(self, *dummy):
         [line, col] = self.cmitFile.textW.getCursorPosition()
-        self.statusBar().message('Column: ' + str(col))
+        self.setStatusBar('Column: ' + str(col))
 
     def loadSettings(self):
         self.splitter.setSizes(settings().splitter)
@@ -206,14 +214,15 @@ class MainWidget(qt.QMainWindow):
         self.cmitFile.listViewItem = self.cmitItem
         
     def about(self, ignore):
-        qt.QMessageBox.about(self, "About " + applicationName,
-"<qt><center><h1>" + applicationName + " " + version + """</h1></center>\n
-<center>Copyright &copy; 2005 Fredrik Kuivinen &lt;freku045@student.liu.se&gt;
-</center>\n<p>
-<center>Copyright &copy; 2005 Mark Williamson &lt;maw48@cl.cam.ac.uk&gt;
-</center>\n<p> This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License version 2 as
- published by the Free Software Foundation.</p></qt>""")
+        str = '<qt><center><h1>%(appName)s %(version)s</h1>' \
+              '<p>Copyright &copy; 2005 Fredrik Kuivinen &lt;freku045@student.liu.se&gt;</p>' \
+              '<p>Copyright &copy; 2005 Mark Williamson &lt;maw48@cl.cam.ac.uk&gt;</p></center>' \
+              '<p>This program is free software; you can redistribute it ' \
+              'and/or modify it under the terms of the GNU General Public ' \
+              'License version 2 as published by the Free Software Foundation.' \
+              '</p></qt>' % {'appName': applicationName, 'version': version}
+
+        qt.QMessageBox.about(self, "About " + applicationName, str)
 
     def contextMenuRequestedSlot(self, item, pos, col):
         if item and not item.commitMsg:
@@ -266,8 +275,8 @@ EDITOR.''')
     def editorExited(self):
         p = self.sender()
         status = p.exitStatus()
-        file = str(p.arguments()[1])
-        editor = str(p.arguments()[0]) + ' ' + file
+        file = unicode(p.arguments()[1])
+        editor = unicode(p.arguments()[0]) + ' ' + file
         if not p.normalExit():
             qt.QMessageBox.warning(self, 'Editor failure',
                                    'The editor, ' +  editor + ', exited abnormally.')
@@ -296,7 +305,14 @@ EDITOR.''')
         self.refreshFiles()
 
     def currentChange(self, item):
-        self.text.raiseWidget(item.file.textW)
+        f = item.file
+        if not f.textW:
+            f.textW = self.newTextEdit()
+            f.textW.setReadOnly(True)
+            f.textW.setTextFormat(Qt.RichText)
+            f.textW.setText(formatPatchRichText(f.getPatch(), self.patchColors))
+
+        self.text.raiseWidget(f.textW)
         self.currentContextItem = item
         if item.commitMsg:
             self.updateCommitCursor()
@@ -314,7 +330,7 @@ EDITOR.''')
             else:
                 keepFiles.append(item.file)
 
-        commitMsg = str(self.cmitItem.file.textW.text())
+        commitMsg = unicode(self.cmitItem.file.textW.text())
 
         if not selFileNames:
             qt.QMessageBox.information(self, "Commit - " + applicationName,
@@ -348,70 +364,68 @@ EDITOR.''')
         ret = FileState()
         cur = self.filesW.currentItem()
         if cur and cur != self.cmitItem:
-            ret.current = self.filesW.currentItem().file.dstName
+            ret.current = self.filesW.currentItem().file.text
         else:
             ret.current = None
         ret.selected = sets.Set()
 
         for x in self.filesW:
             if x.isSelected():
-                ret.selected.add(x.file.dstName)
+                ret.selected.add(x.file.text)
 
         return ret
 
     def restoreFileState(self, state):
         for f in self.files:
-            f.listViewItem.setSelected(f.dstName in state.selected)
+            f.listViewItem.setSelected(f.text in state.selected)
 
         for x in self.filesW:
-            if x.file.dstName == state.current:
+            if x.file.text == state.current:
                 self.filesW.setCurrentItem(x)
 
     def newTextEdit(self):
         ret = qt.QTextEdit()
         self.text.addWidget(ret)
-        return ret
-    
-    def setFiles(self, files):
+        return ret    
+
+    def addFile(self, file):
+        f = file
+        f.listViewItem = MyListItem(self.filesW, f)
+
+        # The patch for this file is generated lazily in currentChange
+
+        # Only display files that match the filter.
+        f.listViewItem.setVisible(self.filterMatch(f))
+
+        self.filesW.insertItem(f.listViewItem)
+
+
+    def removeFile(self, file):
+        f = file
+        self.text.removeWidget(f.textW)
+        self.filesW.takeItem(f.listViewItem)
+        f.listViewItem = None
+
+    def refreshFiles(self):
         state = self.getFileState()
-        self.filesW.clear()
-        self.createCmitItem()
-        for f in self.files:
-            self.text.removeWidget(f.textW)
-            f.listViewItem = None
-
-        self.files = []
-        for f in files:
-            f.textW = self.newTextEdit()
-            f.textW.setReadOnly(True)
-            f.textW.setTextFormat(Qt.RichText)
-            f.textW.setText(formatPatchRichText(f.patch, self.patchColors))
-            self.files.append(f)
-
-            f.listViewItem = MyListItem(self.filesW, f)
-            # Only display files that match the filter.
-            f.listViewItem.setVisible(self.filterMatch(f))
-            self.filesW.insertItem(f.listViewItem)
 
+        self.setUpdatesEnabled(False)
+        scm.updateFiles(self.files)
         self.filesW.setCurrentItem(self.cmitItem)
 
         # For some reason the currentChanged signal isn't emitted
         # here. We call currentChange ourselves instead.
         self.currentChange(self.cmitItem)
-        
         self.restoreFileState(state)
+        self.setUpdatesEnabled(True)
+        self.update()
 
-    def refreshFiles(self, ignored=None):
-        files = scm.getFiles()
-        if settings().quitOnNoChanges and len(files) == 0:
+        if settings().quitOnNoChanges and len(self.files) == 0:
             self.close()
-        else:
-            self.setFiles(files)
-
-        return len(files) > 0
+        return len(self.files) > 0
 
     def filterMatch(self, file):
-        return file.dstName.find(str(self.filter.text())) != -1
+        return file.dstName.find(unicode(self.filter.text())) != -1
         
     def updateFilter(self, ignored=None):
         for w in self.filesW:
@@ -470,14 +484,10 @@ def formatPatchRichText(patch, colors):
             else: style = 'std'
             ret.extend(['</font><font color="', colors[style], '">'])
             prev = c
-        line = qt.QStyleSheet.escape(l).ascii()
-        if not line:
-            line = ''
-        else:
-            line = str(line)
+        line = unicode(qt.QStyleSheet.escape(l))
         ret.extend([line, '\n'])
     ret.append('</pre></qt>')
-    return ''.join(ret)
+    return u''.join(ret)
 
 def getEditor():
     if os.environ.has_key('GCT_EDITOR'):
